<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for customizing migrations in the custom area. If migrations folder is in the custom area these migrations will be processed also
 */
Yii::import('system.cli.commands.MigrateCommand');

class PMigrateCommand extends MigrateCommand {
    /**
     * @var array contains paths to the migration folders
     */
    public $migrationPaths = array();
    /**
     * This method is invoked right before an action is to be executed (after all possible filters.)
     * It checks does the migration path exist, and buildes pathes to the migration folders (common and custom)
     * @param CAction $action the action to be executed.
     * @return boolean whether the action should be executed.
     */
    public function beforeAction($action,$params)
    {
        if ($action !== 'up' && $action !== 'down') {
            if (is_array($this->migrationPath)) {
                $this->migrationPath = $this->migrationPath[0];
            }
            return parent::beforeAction($action, $params);
        }
        $paths = array();

        if (!is_array($this->migrationPath)) {
            $this->migrationPath = array($this->migrationPath);
        }
        foreach ($this->migrationPath as $path) {
            $path = Yii::getPathOfAlias($path);
            if ($path === false || !is_dir($path)) {
                $section = strpos($path, 'common') ? 'Common' : 'Custom';
                echo "Notice: {$section} migration directory does not exist.";
            }
            $paths[] = $path;
        }
        $this->migrationPath = $paths;

        $yiiVersion = Yii::getVersion();
        echo "\nYii Migration Tool v1.0 (based on Yii v{$yiiVersion})\n\n";

        return true;
    }
    /**
     * Action performs migrations from custom and common areas. This method is called when "yiic migrate" performed
     * @param array $arg all the arguments passed into the command
     */
    public function actionUp($args)
    {
        if (is_string($this->migrationPath)) {
            $this->migrationPath = array($this->migrationPath);
        }
        $paths = $this->migrationPath;
        foreach ($paths as $path) {
            $this->migrationPath = $path;

            if ($path === false || !is_dir($path)) {
                continue;
            }

            if (($migrations=$this->getNewMigrations()) === array()) {
                echo "No new migration found. Your system is up-to-date.\n";
                continue;
            }

            $total = count($migrations);
            $step = isset($args[0]) ? (int)$args[0] : 0;
            if ($step > 0) {
                $migrations = array_slice($migrations, 0, $step);
            }

            $n = count($migrations);
            if ($n === $total) {
                echo "Total $n new " . ($n === 1 ? 'migration' : 'migrations') . " to be applied:\n";
            } else {
                echo "Total $n out of $total new " . ($total === 1 ? 'migration' : 'migrations') . " to be applied:\n";
            }

            foreach ($migrations as $migration) {
                echo "    $migration\n";
            }
            echo "\n";

            if ($this->confirm('Apply the above ' . ($n === 1 ? 'migration' : 'migrations') . "?")) {
                foreach ($migrations as $migration) {
                    if ($this->migrateUp($migration) === false) {
                        echo "\nMigration failed. All later migrations are canceled.\n";
                        return;
                    }
                }
                echo "\nMigrated up successfully.\n";
            }
        }
        $this->createProfileFields();
    }

    public function actionDown($args)
    {
        if (is_string($this->migrationPath)) {
            $this->migrationPath = array($this->migrationPath);
        }
        $paths = $this->migrationPath;

        $step = isset($args[0]) ? (int)$args[0] : 1;
        if ($step < 1) {
            echo "Error: The step parameter must be greater than 0.\n";
            return 1;
        }

        if (($migrations = $this->getMigrationHistory($step)) === array()) {
            echo "No migration has been done before.\n";
            return 0;
        }
        $migrations = array_keys($migrations);

        $n = count($migrations);
        echo "Total $n " . ($n === 1 ? 'migration' : 'migrations') . " to be reverted:\n";
        foreach($migrations as $migration)
            echo "    $migration\n";
        echo "\n";

        if ($this->confirm('Revert the above ' . ($n === 1 ? 'migration' : 'migrations') . "?")) {
            foreach ($migrations as $migration) {
                $result = false;
                foreach ($paths as $path) {
                    $this->migrationPath = $path;

                    if ($path === false || !is_dir($path)) {
                        continue;
                    }

                    if ($this->migrateDown($migration) === false) {
                        continue;
                    }
                    else {
                        $result = true;
                        break;
                    }
                }

                if ($result === false) {
                    echo "\nMigration failed. All later migrations are canceled.\n";
                    return 2;
                }

            }
            echo "\nMigrated down successfully.\n";
        }
    }

    protected function migrateDown($class)
    {
        if ($class === self::BASE_MIGRATION) {
            return;
        }

        if (!file_exists($this->migrationPath.DIRECTORY_SEPARATOR.$class.'.php')) {
            return false;
        }

        echo "*** reverting $class\n";
        $start = microtime(true);
        $migration = $this->instantiateMigration($class);
        if ($migration->down() !== false) {
            $db = $this->getDbConnection();
            $db->createCommand()->delete($this->migrationTable, $db->quoteColumnName('version').'=:version', array(':version' => $class));
            $time = microtime(true) - $start;
            echo "*** reverted $class (time: " . sprintf("%.3f",$time) . "s)\n\n";
        } else {
            $time = microtime(true) - $start;
            echo "*** failed to revert $class (time: " . sprintf("%.3f", $time) . "s)\n\n";
            return false;
        }
    }

    /**
     * The method checks do the fields listed in
     * the profileFields table exist in the Profile table.
     * If some field does not exist , it will be created.
     */
    public function createProfileFields()
    {
        Yii::app()->getModule('user');
        $models = ProfileField::model()->findAll();
        $scheme = get_class(Yii::app()->db->schema);
        $tableName = Profile::model()->tableName();
        echo "Checking compliance of the profile fields ...";
        foreach ($models as $model) {

            $existsSql = "SHOW COLUMNS FROM {$tableName} LIKE :varname";
            $exists = count(Yii::app()->db->createCommand($existsSql)
                ->bindValues(array(
                ':varname' => $model->varname
            ))
                ->queryAll());

            if (!$exists) {
                $sql = 'ALTER TABLE ' . Profile::model()->tableName() . ' ADD `' . $model->varname . '` ';
                $sql .= str_replace('UNIX-DATE', 'INTEGER', $model->field_type);
                if (
                    $model->field_type !== 'TEXT'
                    && $model->field_type !== 'DATE'
                    && $model->field_type !== 'BOOL'
                    && $model->field_type !== 'BLOB'
                    && $model->field_type !== 'BINARY'
                ) {
                    $sql .= '(' . $model->field_size . ')';
                }
                $sql .= ' NOT NULL ';

                if ($model->field_type !== 'TEXT' && $model->field_type !== 'BLOB' || $scheme !== 'CMysqlSchema') {
                    if ($model->default) {
                        $sql .= " DEFAULT '" . $model->default . "'";
                    } else {
                        if (
                            $model->field_type === 'TEXT'
                            || $model->field_type === 'VARCHAR'
                            || $model->field_type === 'BLOB'
                            || $model->field_type === 'BINARY'
                        ) {
                            $sql .= " DEFAULT ''";
                        } else if ($model->field_type === 'DATE') {
                            $sql .= " DEFAULT '0000-00-00'";
                        } else {
                            $sql .= " DEFAULT 0";
                        }
                    }
                }
                Yii::app()->db->createCommand($sql)->execute();
                echo "{$model->varname} is added into {$tableName} table\n";
            }
        }
    }
}
