<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for working with Gentics Content Connector
 */
class TagcloudApi extends CComponent
{

    /**
     * @var array $folderIds contains folder IDs of content to show
    */
    public $folderIds;

    /**
     * @var string $lang contains current languagecode
     */
    public $lang;

    /**
     * array that contains received data
     * @var $data
     */
    public $data = array();

    /**
     * array that contains extracted tags
     *
     * @var $tags
     */
    public $tags = array();

    /**
     * @var bool $usePersonalisation defines if personalisation should be performed
     */
    public $usePersonalisation = true;

    /**
     * @var string $htmlCacheKey contains cacheKey that can be used for HTML caching
     */
    public $htmlCacheKey = '';

    /**
     * @var string $overviewPage CMS page with overview widget
     */
    public $overviewPage;

    /**
     * @var string tagstyles
     */
    public $tagstyles = array();

    /**
     * @var string tagsSeparator
     */
    public $tagsSeparator = ', ';

    /**
     * Performs GCC request, filters data by personalisation rules and extacts tags from received data
     *
     * @return void|bool
     */
    public function run($limit = null)
    {
        //Yii::app()->cache->flush();

        if (!empty($this->folderIds)) {
            sort($this->folderIds);

            $this->_loadData();
            $this->_cleanUp(Yii::app()->user->id);
            $this->_extractTags($limit);

            return true;

        } else {
            return false;
        }
    }

    /**
     * Extracts tags from loaded data
     *
     * @return bool
     */
    private function _extractTags($limit = null)
    {
        if (empty($this->data)) {
            return false;
        }

        $this->tags = array();

        foreach ($this->data as $entry) {
            if (!empty($entry['attributes'][Yii::app()->getModule('tagcloud')->tagging_name])) {
                $tags = explode($this->tagsSeparator, $entry['attributes'][Yii::app()->getModule('tagcloud')->tagging_name]);

                foreach ($tags as $tag) {
                    $tag = trim($tag);
                    $this->tags[$tag] = empty($this->tags[$tag]) ? 1 : $this->tags[$tag]+1;
                }
            }
        }

        Yii::trace('Tags and usage: '.print_r($this->tags, true), 'Api requests(start)');

        if (!empty($this->tags)) {
            
            arsort($this->tags);
            reset($this->tags);

            if(isset($limit)) {
                $this->tags = array_slice($this->tags, 0 ,$limit);
            }

            $tagstylesQty = count($this->tagstyles);
            $max = max($this->tags);
            $step = ceil($max / count($this->tagstyles));

            foreach ($this->tags as $k=>$v) {
                $index = round($v/$step);
                $index = $index<=($tagstylesQty-1) ? $index : $index-1;

                $url = $this->overviewPage;
                $url .= strpos($url, '?')!==false ? '&tag='.$k : '?tag='.$k;
                $url .= '&folderIds='.str_replace('10002.', '', implode(',', $this->folderIds));

                $this->tags[$k] = array(
                    'class' => $this->tagstyles[$index],
                    'url' => $url,
                );
            }
        }

        return true;
    }

    /**
     * Filter data by some schema
     *
     * @param integer $user_id Id of the user to personalize tree for
     *
     * @return array
     */
    private function _cleanUp($user_id)
    {
        $maxTimestamp = $this->loadMaxUpdatetimestamp();
        $cacheId = 'tagcloud_PERS_'.md5(implode(',', $this->folderIds).$this->lang.$maxTimestamp.intval($this->usePersonalisation));
        $this->htmlCacheKey = 'tagcloud_HTML_'.md5(implode(',', $this->folderIds).implode(',', $this->tagstyles).$this->lang.$maxTimestamp.intval($this->usePersonalisation).$this->overviewPage.$this->tagsSeparator);

        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            //get personalisation fields from settings
            $filters = $this->getFiltersList();
            $data = array();

            // apply filter here
            if ($filters && $this->data) {
                foreach ($this->data as $id=>$contentPage) {
                    if (!is_null($this->_unsetNodes($filters, $user_id, $contentPage))) {
                        $data[$id] = $contentPage;
                    }
                }
                $this->data = $data;
            }

            

            Yii::app()->cache->set($cacheId, $this->data);
        } else {
            $this->data = $data;
        }
    }

    /**
     * Private method that will unset not allowed data
     *
     * @param array $filters contains keys that will be used as filters
     * @param array $user_id contains user permissions
     * @param array $data    points to the array with nodes tree
     *
     * @return array|null
     */
    private function _unsetNodes($filters, $user_id, $data)
    {
        /*  check permissions for current node */
        $persAttributes = array();
        foreach ($filters as $filter) {
            if (!empty($data['attributes'][$filter])) {
                $persAttributes = array_merge($persAttributes, $data['attributes'][$filter]);
            }
        }

        // Call function that will check permissions
        if ($this->usePersonalisation) {
            if (!Yii::app()->getModule('personalisation')->rule->checkAccess($user_id, $persAttributes)) {
                /* if user does not have permissions - unset the node */
                return null;
            }
        }

        return $data;
    }

    /**
     * Filter tree by some schema
     *
     * @return array|bool
     */
    public function getFiltersList()
    {
        if (Yii::app()->getModule('contentSource')->contentSource->usePersonalisation) {
            $filters = Yii::app()->getModule('contentSource')->contentSource->personalisationFields;
            return $filters;
        }

        return false;
    }

    /**
     * Load required tree data from cache. If not exists request Gentics Content Connector API for a new tree.
     *
     * @return array
     */
    private function _loadData()
    {
        $data = array();
        $filtersList = '';
        $requertFields =  array(
            'edittimestamp',
            Yii::app()->getModule('tagcloud')->tagging_name
        );

        // get max timestamp for given folderIds
        $maxTimestamp = $this->loadMaxUpdatetimestamp();

        // create cache key that includes folderIds+language+gccRequestString+timestamp of last updated folder;
        $cacheId = 'tagcloud_'.md5(implode(',', $this->folderIds).$this->lang.$maxTimestamp);

        // Load data
        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            $data = Yii::app()->repositoryApi->request(
                array(
                    'filter' => 'object.obj_type==10007 AND object.languagecode=="'.$this->lang.
                    '" AND (object.folder_id=="'.implode('" OR object.folder_id=="', $this->folderIds).'")',
                    'type' => 'php',
                ),
                CMap::mergeArray($this->getFiltersList(), $requertFields)
            );
            $data = unserialize($data);

            if (isset($data['status']) && $data['status'] == 'ok') {
                unset($data['status']);
            } else {
                $data = array();
            }
            Yii::app()->cache->set($cacheId, $data);
        }

        $this->data = $data;
    }

    /**
     * extracts max updatetimestamp from array of CMS objects
     *
     * @return int|bool
     */
    public function loadMaxUpdatetimestamp()
    {
        // TODO cachekey is wrong - should only be cached for $cacheTime
        $timestampCacheId = 'tagcloud_UTS_'.md5(implode(',', $this->folderIds).$this->lang);

        //try to get $maxTimestamp from cache
        if (($maxTimestamp = Yii::app()->cache->get($timestampCacheId)) === false) {
            $timestamps = Yii::app()->repositoryApi->request(
                array(
                    'filter' => 'object.obj_type==10007 AND object.languagecode=="'.$this->lang.
                    '" AND (object.folder_id=="'.implode('" OR object.folder_id=="', $this->folderIds).'")',
		     'type' => 'php',
                    'sorting' => 'updatetimestamp:desc',
                    'count' => '1'

                ),
                array('updatetimestamp')
            );
            $timestamps = unserialize($timestamps);

            if (isset($timestamps['status']) && $timestamps['status']=='ok') {
                unset($timestamps['status']);
                $maxTimestamp = 0;
                foreach ($timestamps as $id=>$contentPage) {
                    if (isset($contentPage['attributes']['updatetimestamp']) && $contentPage['attributes']['updatetimestamp'] > $maxTimestamp) {
                        $maxTimestamp = $contentPage['attributes']['updatetimestamp'];
                    }
                }

                Yii::app()->cache->set($timestampCacheId, $maxTimestamp, Yii::app()->getModule('tagcloud')->cacheTime);
            } else {
                $maxTimestamp = false;
            }
        }
        return $maxTimestamp;
    }
}
